#-*-coding:utf-8-*-

"""
fournit la classe Zisch qui, comme son nom l'indique,
produit des bruits de sifflements bruités.
"""

import numpy as NY
from samplerate import sr
from conversions08 import extremum

import profils as _p
import filtres as _f
import canal
import serie
from origine import Origine


__author__ = "Copyright (c) 2005-07 René Bastian (Wissembourg, FRANCE)"
__date__ = "2007.04.30"


class Zisch(Origine):
    """
    classe pour produire des sifflements bruités.
    """
    def __init__(self, filtre=None, couple=(3452341, 756451)):
        """
        classe pour obtenir un bruit filtré par une bande
        dont la fréquence centrale et la largeur est variable.
        Il faudrait faire en sorte que
        - le type de filtre (récursif, non-récursif)
        - le nombre de coefficients puisse être choisi
        (il faut donc une batterie de filtres dont l'appel
        se fait tel que resultat = filtre(signal, vecteur_de_coeffs)
        où vecteur_de_coeffs = ((a,b),(a,b),(a,b),(a,b),(a,b),(a,b), ...)
        et a = (a0, a1, ...), b = (b0, b1, ...)²
        """
        Origine.__init__(self)
        self.alea = serie.Serie002(*couple)
        if filtre == None:
            self.filtre = _f.appliBPrecursifPronyVar
        else:
            self.filtre = filtre
            
    def __call__(self, duree, profilProny, profilLargeur,
                 profilEnv, profilPano):
        """
        retourne un bruit profilé. 
        'profilProny' : profil de la variation de la hauteur centrale
        du filtre
        'deltas' : largeur = (prony-delta .. prony+delta)
        """
        n = int(sr * duree)
        w = self.alea.uniform(-1., +1., n)
        pronys = _p.ProfilBase(profilProny).samples(n)
        deltas = _p.ProfilBase(profilLargeur).samples(n)

        liste = NY.array([pronys, deltas])
        liste.transpose()
        w = self.filtre(w, liste)
        n = len(w)
        env = _p.ProfilBase(profilEnv).samples(n)
        pano = _p.ProfilBase(profilPano).samples(n)
        w /= extremum(w)
        w *= env
        wg, wd = w * pano, w * (1.0-pano)
        return wg, wd

    def frappe(self, duree, profilProny, profilLargeur,
                 profilEnv, profilPano):
        """
        à mesure qu'on s'approche de l'extrême grave (prony=12.)
        le son devient très percutant
        """
        wg, wd = self.__call__(duree, profilProny, profilLargeur,
                 profilEnv, profilPano)
        cg, cd = canal.CanalMono(), canal.CanalMono()
        cg.addT(0.0, wg)
        cg.autoreverb()
        cd.addT(0.0, wd)
        cd.autoreverb()
        return cg.ex(), cd.ex()


