#-*-coding:utf-8-*-

"""
Avec 'py2tricks.py' on peut produire un fichier en format
Latex/Pstricks pour représenter un vecteur 'y' de nombres 'x'.
Ce n'est oas une grosse machine qu'on peut paramétrer pour
tous les cas, mais d'un petit script que l'utilisateur doit
adapter à son cas (ce qui est déjà visible dans la chaîne 'tete').
En outre, si on choisit n'importe quoi comme dimensions
de l'image, on obtient n'importe quoi: il faut que ce qu'on
veut avoir comme image soit adapté aux données et cela se
passe dans la ligne:
'x0=0.0, y0=-5.0, x1=10.0, y1=5.0' de l'instanciation.
Remerciements et coups de chapeau:
L'objet 'showgrid' vient du 'Latex Graphics Companion'
de M. Goossens, S. Rahtz et F. Mittelbach;
'PSTricks' a été écrit par Timothy van Zandt,
'Latex' par L. Lamport et 'TeX' par D. E. Knuth.
"""


__author__ = "Copyright (c) 2006-09 René Bastian (Wissembourg, FRANCE)"
__date__ = "2009.02.14"

tete = """
\documentclass[b5paper]{article}
\usepackage[latin1]{inputenc}
\usepackage[T1]{fontenc}
\usepackage[frenchb]{babel}
%\usepackage{amsmath}
\usepackage[xcolor=pst]{pstricks}
%\usepackage{graphicx, color}
%\usepackage{hyperref}
\\begin{document}
\\newpsobject{showgrid}{psgrid}{subgriddiv=1,griddots=10,gridlabels=6pt}\n
"""

coda = """
\end{document}\n
"""

class Py2Pstricks(object):
    """
    représenter les valeurs d'un vecteur en pstricks
    p.ex. un profil ou une onde
    """
    def __init__(self, titre=None, 
                 x0=0.0, y0=-5.0, x1=10.0, y1=5.0):
        """créer une instance
        'axeXs, axeYs' : annotation des axes
        'ecartX' : écart graphique entre les valeurs 
        'titre' : titre du graphe
        """
        self.p0 = "(%f,%f)" % (x0, y0)
        self.p1 = "(%f,%f)" % (x1, y1)
        self.x0, self.x1, self.y0, self.y1 = x0, x1, y0, y1
        self.marge = 0.5
        self.axe = None
        self.dx = None 
        self.titre = titre
        self.xini, self.xfin, self.yinf, self.ysup = None, None, None, None
        self.linewidth = 3.0 
        self.linestyle = "solid" 
        self.showpoints = "true" 
        self.linecolor = None 
        self.calculs()

    def calculs(self):
        """
        un peu de ménage
        """
        self.p0 = "(%f,%f)" % (self.x0, self.y0)
        self.p1 = "(%f,%f)" % (self.x1, self.y1)
        self.axe = (self.y1 - self.y0)/2
        
        self.xini = self.x0 + self.marge
        self.xfin = self.x1 - self.marge
        self.yinf = self.y0 + self.marge
        self.ysup = self.y1 - self.marge

    def attributs(self):
        """
        intègre les valeurs données aux attributs des tracés
        """
        r = []
        r.append("\psset{linewidth=%6.3fpt}\n" % self.linewidth)
        r.append("\psset{linestyle=%s}\n" % self.linestyle)
        r.append("\psset{showpoints=%s}\n" % self.showpoints)
        return "".join(r)

    def barres(self, liste):
        """
        au lieu d'une ligne continue, des traits
        qui partent de la ligne y=0.0
        """
        self.calculs()
        
        n = len(liste)
        self.dx = (self.xfin - self.xini)/(n + 1)
        ss = "\\begin{pspicture}%s%s\showgrid\n" % (self.p0, self.p1)
        r = [ss]
        ss = "\psframe[linewidth=2pt]%s%s\n" % (self.p0, self.p1)
        r.append(ss)
        ss = self.attributs()
        r.append(ss)
        diffy = max(liste) - min(liste)
        try:
            coeffy = (self.ysup - self.yinf) / diffy
        except ZeroDivisionError:
            print "Tous des nuls !"
            return
        except TypeError:
            coeffy = 1.0
        x = self.xini
        for y in liste:
            ycrt = y * coeffy 
            pini = "(%6.3f,%6.3f)" % (x, 0.0)
            pfin = "(%6.3f,%6.3f)" % (x, ycrt)
            p = "".join(["\psline", pini, pfin, "\n"])
            r.append(p)
            x += self.dx
            if x > self.xfin:
                print "lignes:dépasst x...", self.xfin, x, self.x0, self.x1
            
            if ycrt > self.ysup or ycrt < self.yinf:
                print "lignes:dépasst y...", self.yinf, "...", self.ysup, ":",
                print ycrt, self.y0, self.y1
        r.append("\end{pspicture}\n")
        
        if self.titre != None:
            stitre = "\\begin{center}\\textbf{%s}\end{center}\n" % (self.titre)
            r.append(stitre)
        s = "".join(r)
        return s

    def lignes(self, liste):
        """
        envoie la représentation de 'y' dans le fichier 'nomf'
        """
        self.calculs()
        n = len(liste)
        self.dx = (self.xfin - self.xini)/(n + 1)
        ss = "\\begin{pspicture}%s%s\showgrid\n" % (self.p0, self.p1)
        r = [ss]
        ss = "\psframe[linewidth=%5.2fpt]%s%s\n" % (self.dx, self.p0, self.p1)
        r.append(ss)
        ss = self.attributs()
        r.append(ss)
        r.append("""\psline{-}""")
        diffy = max(liste) - min(liste)
        try:
            coeffy = (self.ysup - self.yinf) / diffy
        except ZeroDivisionError:
            print "lignes:Tous des nuls !"
            return
        except TypeError:
            coeffy = 1.0
        x = self.xini
        for y in liste:
            ycrt = y * coeffy 
            p = "".join(["(", str(x), ",", str(ycrt), ")"])
            r.append(p)
            x += self.dx
            if x > self.xfin:
                print "lignes:dépasst x...", self.xfin, x, self.x0, self.x1
            if ycrt > self.ysup or ycrt < self.yinf:
                print "lignes:dépasst y...", self.yinf, "...", self.ysup, ":",
                print ycrt, self.y0, self.y1
        r.append("\end{pspicture}\n")
        
        if self.titre != None:
            stitre = "\\begin{center}\\textbf{%s}\end{center}\n" % (self.titre)
            r.append(stitre)
        s = "".join(r)
        return s

if __name__ == "__main__":
    from serie import Serie002
    import os

    py2ps = Py2Pstricks()
    v = Serie002(1357, 140209).uniform(-1., 1., 12)
    rtexte = [tete]

    py2ps.titre = "Exemple de lignes 1"
    py2ps.linewidth = 2.1
    py2ps.dimY = 4.0    
    rtexte.append(py2ps.lignes(v))

    py2ps.titre = "Exemple de barres 1"
    py2ps.dimY = 5.0
    rtexte.append(py2ps.barres(v))

    v = Serie002(1357, 140209).uniform(-1., 1., 148)
    py2ps.titre = "Exemple de lignes 2"
    py2ps.x0, py2ps.y0 = -1.0, -6.
    py2ps.x1, py2ps.y1 = 15.0, 6.
    rtexte.append(py2ps.lignes(v))

    py2ps.titre = "Exemple de barres 2"
    rtexte.append(py2ps.barres(v))


    rtexte.append(coda)
    texte = "\n".join(rtexte)
    nomf = "test_py2ps"
    open(nomf+".tex", "w").write(texte)
    os.system("latex " + nomf)
    os.system("dvips " + nomf)
    os.system("gv " + nomf + ".ps")
    


