#-*-coding:utf-8-*-

"""
fournit la classe FiltreBPVariable (jonc de mer) et calcule les
coefficients du filtre par vecteurs complets.
"""

__author__ = "Copyright (c) 2005 René Bastian (Wissembourg, FRANCE)"
__date__ = "2006.08.28, 10.17, 08.10.06, 09.02.08"

from samplerate import sr
from conversions08 import extremum, prony2freq
from Profils import ProfilBase
import ajuster as _a
import numpy as NY
from origine import Origine


class FiltreBPVariable(Origine):
    """
    classe de filtre passe-bande dont la hauteur et la largeur
    varient selon un profil en 'prony'.
    """
    def __init__(self):
        """
        initilalise mais ne fait rien d'autre
        """
        Origine.__init__(self)
        self.coeffs = None 

    def calcul_coeffs(self, freqcoupure, bandepassante):
        """        
        calcule les coefficients (a0, a1, a2), (0.0, b1, b2)
        d'un filtre BP récursif caractérisé par
        sa fréquence centrale et sa bande passante
        'freqcoupure' : fréquence centrale
        'bandepassante' : bande passante

        source: cf.Steven W. Smith p.326
        """
        bande = bandepassante * 1.0/sr
        fc = freqcoupure * 1.0/sr 
        omega = 2 * NY.pi * fc
        cosomega = NY.cos(omega)
        R = 1.0 - 3.0 * bande
        R2 = R * R
        K = (1.0 - 2 * R * cosomega + R2)/(2.0 - 2 * cosomega)
        a0 = 1.0 - K
        a1 = 2 * (K - R) * cosomega 
        a2 = R2 - K
        b1 = 2 * R * cosomega
        b2 = -R2
        self.coeffs = [a0, a1, a2, b1, b2]
        self.coeffs = NY.transpose(self.coeffs)

    
    def appli(self, x):
        """
        appli applique une liste de coeffs de filtres généraux d'ordre 2
        'x' : signal entrant
        """
        d = min(len(x), len(self.coeffs))
        x = NY.concatenate((NY.zeros(2, NY.float64), x))
        y = NY.zeros(len(x), NY.float64)
        for i in range(2, d-2):
            a0, a1, a2, b1, b2 = self.coeffs[i]
            y[i] = x[i] * a0 + x[i-1] * a1 + x[i-2] * a2 + \
                    y[i-1] * b1 + y[i-2] * b2
        return y
    
    def __call__(self, signal, vfreq, vlargeur):
        """
        'signal' : un signal
        'vfreq' : profil d'une variation de fréquence centrale en Hz
        'vlargeur' : profil d'une variation de largeur de bande en Hz
        """
        
        vfreq, vlargeur = _a.prolongerN(vfreq, vlargeur)
        
        self.calcul_coeffs(vfreq, vlargeur)
        y = self.appli(signal)
        return y
        return y / extremum(y)

    def evt(self, signal, profilHauteurs, profilLargeurs):
        """
        'signal' : le signal à filtrer
        'profilHauteurs, profilLargeurs' : listes-profils
        décrivant la variation de la fréquence centrale et
        la largeur du filtre en pronys.
        """
        n = len(signal)
        
        
        
        
        pH = ProfilBase(profilHauteurs).samples(n)
        pH = prony2freq(pH)
        pL = ProfilBase(profilLargeurs).samples(n)
        pL = prony2freq(pL)
        
        return self(signal, pH, pL)

if __name__  == "__main__":
    import serie
    from Profils import recta
    import live
    import sys
    import time


    audio = live.Audio()
    pp = pprint.PrettyPrinter(indent=4)
    nomaudio = "".join(["p", __file__[:-3]])
    
    def test1():
        "test 2: "
        nomf = "_".join([__file__[:-3], "test2"])
        print "Test", ntest, nomf
        fbp = FiltreBPVariable()
        pp.pprint(fbp)

        xsignal = serie.Serie002().bruit(3*4*sr)
        xH = [[recta, 1., 32., 88.]]
        xL = [[recta, 1., 3., 0.01]]
        t0 = time.time()
        w = fbp.evt(xsignal, xH, xL)
        t1 = time.time()
        print "Durée", t1 - t0, len(w)
        audio.play(w, w)
     
   


