#-*-coding:utf-8-*-

import os
import numpy as NY

import filtres as _f
import decibel
import utiles as _u
import profils as _p
import ondes as _o
import canal
import serie
import sys
from origine import Origine


class PercuEthnique2(Origine):
    def __init__(self, ecoute = True, coeff = 0.999,  
                 couple=(2458, 4731)):
        """ 
        propose un filtrage par filtre en peigne pour obtenir 
        des sons changeant de timbre
        """
        Origine.__init__(self)
        self.alea = serie.Serie002(*couple)
        self.coeff = coeff
        
        self.ecoute = ecoute
        self.db = decibel.Decibel()
        self.Info = []

    def evt(self, prony, fprony, duree, coeff=None, crit=1):
        " comme __call__"
        return self(prony, fprony, duree, coeff, crit)

    def __call__(self, prony, fprony, duree, coeff=None, crit=1):
        """
        'prony' : hauteur de base 
        'fprony' : hauteur à filtrer (normalement: fprony > prony)
        'coeff' : coefficient du filtre 
        'duree' : durée du signal à produire 
        'crit=1' : si l'amorce est entièrement du bruit

        Variante:
        - 'coeff' est un vecteur donc 'duree' est enlevé
        - 'fprony' est un vecteur donc 'duree' est enlevé
        """
        if fprony < prony:
            s = "PercuEthnique2(prony, fprony): fprony < prony: Filtrage inutile"   
            print s
        if coeff is None:
            coeff = self.coeff 
        if self.ecoute:
            alea_str = str(self.alea)
        nprony = _u.prony2samples(prony) 
        nfprony = _u.prony2samples(fprony) 
        nfinal = int(duree * _u.sr) + nfprony + nprony
        c = NY.zeros(nfinal, NY.float64)
        if crit == 1: 
            amorce = self.alea.bruit(nprony + nfprony)            
            c[0:nprony+nfprony] = amorce
        else: 
            amorce = self.alea.bruit(nprony)
            c[nfprony:nfprony+nprony] = amorce

        i = nprony + nfprony
        j = nfprony
        r0 = 1.
        r = 0.999
        for i in range(nprony+nfprony, nfinal):            
            j = i - nprony
            c[i] = c[j] - coeff * c[j-nfprony]
            if i % nprony == 0:
                r0 *= r
            c[i] *= r0
        w = c[nprony+nfprony:]

        w = w / _u.extremum(w)
        
        
        if self.ecoute:
            pass
            
        return w


class PercuEthnique1(Origine):
    """ 
    classe produisant des sons de contre-basse pincée ou frappée dans le grave
    et des sons métalliques dans le médium et aigu.
    """
    def __init__(self, ecoute = True, nc = 15, decalprony=0., 
                 couple=(2458, 4731)):
        """
        'decalprony' : decalage du filtre par rapport à la
        hauteur voulue
        'nc' : nombre de coefficients pour le filtre
        'ecoute' : True si on veut écouter le son individuellement 
        'couple' : nombres initiaux pour le générateur sériel.
        """
        Origine.__init__(self)
        self.alea = serie.Serie002(*couple)
        self.nc = nc
        self.fpb = _f.FiltreFixe(genre="pb", nc=nc)
        self.ecoute = ecoute
        self.db = decibel.Decibel()
        self.Info = []
        
        self.dprony = decalprony

    def __call__(self, prony, dynam, pano, r0):
        """
        'prony' : hauteur 
        'dynam' : intensité 
        'pano' : lieu stéréophonique 
        'r0' : amortissement
        """        
        c = canal.CanalMono()            
        if self.ecoute:
            alea_str = str(self.alea)
        amorce = self.alea.noise(_u.prony2periode(prony))
        
        na = len(amorce)
        c.addT(0.0, amorce)               
        r = 1.0
        i = na        
        while r > _u.epsilon:
            r *= r0
            signal = c.a[i-na:i-1]            
            w = self.fpb(signal, prony + self.dprony)
            w *= r
            c.addN(i, w)                    
            i += na        
        ampli = self.db(dynam)
        w = c.ex()/ _u.extremum(c.ex())
        
        w = self.fpb(w, 120.)
        w = w[na:]        
        w *= ampli                                
        wg, wd = w * pano, w * (1.-pano) 
        if self.ecoute:
            pass
        return wg, wd

    def evt(self, prony, duree, pronyP, amortiR):
        """
        'prony' : hauteur prony initiale 
        'duree' : durée 
        'pronyP' : profil-liste de la variation du filtre         
        'amortiR' : profil-liste de la variation de l'amortissement
        """
        c = canal.CanalMono()            
        if self.ecoute:
            alea_str = str(self.alea)
        amorce = self.alea.noise(_u.prony2periode(prony))        
        na = len(amorce)
        c.addT(0.0, amorce)
        nsignal = int(duree * _u.sr)
        nondes = (nsignal / na) + 1        
        pr = _p.ProfilBase(amortiR).samples(nondes)
        pfprony = _p.ProfilBase(pronyP).samples(nondes)
        i = na        
        for r, fprony in zip(pr, pfprony): 
            signal = c.a[i-na:i-1]            
            w = self.fpb(signal, fprony)
            w *= r
            c.addN(i, w)                    
            i += na        
        
        w = c.ex()/ _u.extremum(c.ex())
        
        
        w = w[na:]        
        
        
        if self.ecoute:
            audio.play(wg, wd)
        return wg, wd
    

