#-*-coding:utf-8-*-

"""
Le problème de la correspondance temps/échantillons est insoluble.  Il
faut ajuster les longueurs des vecteurs par complétion ou amputation.
En cas de complétion on peut compléter avec des zéros ou la dernière
valeur.  Comme la différence ne joue en général que sur qqs
échantillons, je ne prévois pas d'extrapolation.  L'amputation n'a pas
été utilisée - donc j'expulse la fonction.
"""

__author__ = "Copyright (c) 2005 René Bastian (Wissembourg, FRANCE)"
__date__ = "2006.07.07 Rochefort s/Loire, 2008.03.13 St-Mamert",

import numpy as NY

__all__ = ["allonger", "prolongerZ", "prolongerN"]
           
"""
'amputer', 'entrelacer', 'prolongerV' sont transférés dans 'zajuster.py'
"""

def prolongerZ(*vecteurs):
    """ajoute des zéros aux vecteurs donc la
    dimension est inférieure au plus grand vecteur."""
    n = len(vecteurs[0])
    for v in vecteurs[1:]:
        if len(v) > n:
            n = len(v)
    C = []
    for v in vecteurs:
        k = n - len(v)
        if k > 0:
            z = NY.zeros(k)
            v = NY.concatenate((v, z))
        C.append(v)
    return tuple(C)

def prolongerN(*vecteurs):
    """ajouter à chaque vecteur autant de fois la
    dernière valeur pour que tous les vecteurs aient
    la dimension du plus grand vecteur."""
    n = len(vecteurs[0])
    for v in vecteurs[1:]:
        if len(v) > n:
            n = len(v)
    C = []
    for v in vecteurs:
        k = n - len(v)
        if k > 0:
            z = NY.ones(k)* v[-1]
            v = NY.concatenate((v, z))
        C.append(v)
    return tuple(C)



