#-*-coding:utf-8-*-

"""interpolation selon Newton et quelques autres
date : 3.08.05
"""
import numpy as NY


from samplerate import sr
from conversions08 import extremum


__author__ = "Copyright (c) René Bastian <rbastian@free.fr>"
__date__ = "2005.08.03, 2006.02.10"


def interpolation_lineaire(y, n):
    """ajoute entre deux valeurs successives de y n valeurs
    en variation lineaire.
    'y : suite de valeurs
    'n : entier    
    """
    N = len(y)
    z = NY.zeros((N-1)*n, NY.float64)
    for i in range(1, N):
        w = NY.arange(y[i-1], y[i], (y[i]-y[i-1])/n) 
        j = (i-1)*n
        z[j:j+n] = w
    return z

def _newtonprofil(t, A, X):
    """
    fonction privée
    """
    def produit(t, a, X):
        """
        fonction privée
        """
        z = a
        for x in X:
            z *= (t-x)
        return z
    
    r = A[0]
    for i in range(1, len(A)):
        v = produit(t, A[i], X[:i])
        r = r + v
    return r

def _Fnewton(x, y, p):
    """
    fonction privée
    """
    n = len(y)
    d = len(x)-len(y)
    z = []
    for i in range(1, n):
        
        a = (y[i]-y[i-1])/(x[i+p]-x[i+p-d-1])
        z.append(a)
    return z

def _Gnewton(X, Y):
    """
    fonction privée
    """
    A = [Y[0]]
    k = len(X)
    p = 0
    while k > 1:
        z = _Fnewton(X, Y, p)
        A.append(z[0])
        k -= 1
        
        Y = z
        p += 1
    return A

def _reformNewton(R):
    "fournit une liste de x et une liste de y" 
    n = len(R)
    return [R[i][0] for i in range(n)], [R[i][1] for i in range(n)]

def profilnewton(R):
    """R est une liste de couples [xi, yi]
    _reformNewton en extrait les x et les y."""
    R = _reformNewton(R)
    X = R[0]
    Y = R[1]
    A = _Gnewton(X, Y)
    T = NY.arange(X[0], X[-1], 1.0 / sr)
    return _newtonprofil(T, A, X)

def profilnewton_uniforme(y, xmax):
    """
    'y : suite de valeurs
    'xmax :nombre de valeurs à intercaler
    """
    n = len(y)
    x = NY.arange(0., xmax, xmax/n)
    a = [x, y]
    a = NY.transpose(a)
    return profilnewton(a) 

class ProfilNewton(object):
    """En fournissant une liste de points
    [[x0, y0], ... [xi, yi], ...]
    on obtient une courbe passant par ces points.
    Le profil est calculé pour tenir entre -1.0 et +1.0 ;
    cela amortit les dépassements inévitables avec ce
    procédé.    
    """
    def __init__(self, listeR = None):
        "fournit une instance de la classe."
        if not listeR:
            self.listeR = listeR
        self.profil = None
        if listeR:
            self._calcul()
    
    def _calcul(self):
        "calcul interne du profil"
        P = self._reformNewton()
        X = P[0]
        Y = P[1]
        A = _Gnewton(X, Y)
        T = NY.arange(X[0], X[-1], 1.0 / sr)
        self.profil = _newtonprofil(T, A, X)
        self.profil /= extremum(self.profil)
        
        

    def _reformNewton(self):
        "fournit une liste de x et une liste de y" 
        n = len(self.listeR)
        return [self.listeR[i][0] for i in range(n)], \
            [self.listeR[i][1] for i in range(n)]

    def evt(self, R):
        """calcul et fourniture du profil."""
        self.listeR = R
        self._calcul()
        return self.profil

    def __call__(self, R):        
        """calcul et fourniture du profil."""
        self.listeR = R
        self._calcul()
        return self.profil


