#-*-coding:utf-8-*-

"""
fournit la classe ProfilBase pour faciliter
l'utilisation des profils.
"""


import numpy as NY

import profils as _p
import profils2 as _p2
import samplerate as _sr
from types import IntType, LongType, FloatType, TupleType
from origine import Origine



class ProfilBase(Origine):
    """
    classe pour adapter un profil donné sous forme de liste
    à une durée ou une longueur souhaitée.
    Si le profil-liste comporte des parties invariantes, données
    sous forme de tuple '(x,)', il faut envisager deux choix :
    - les durées invariantes sont ajoutées à la durée à obtenir
    - les durées invariantes sont comprises dans la durée à obtenir.

    Les parties invariantes sont (uniquement ?) intéressantes pour
    des enveloppes de sons dont on veut que certaines phases
    transitoires aient des durées fixes. Mais aussi pour des
    variations de hauteurs initiales et finales fixes, tandis que
    la partie centrale reste constante ou varie proportionnellement.

    Il se peut que les fonctions 'rafist' et 'bouts' soient plus
    faciles à utiliser que les parties fixes - sauf que les
    parties fixes se mettent automatiquement.
    """
    def __init__(self, unProfil):
        """
        'unProfil': profil-liste == profil donné sous forme de liste
        """
        Origine.__init__(self)
        self.listeP = unProfil
        self.duree = None
        self.listePcrt = unProfil

    def temps(self, duree, transpo=None):
        """
        calcule le profil-vecteur à partir du
        profil-liste de telle façon que le vecteur ait une
        durée de 'duree + parties_fixes'.
        'transpo' : un couple (valeur, operation) où operation = ("*"|"+"|"/"|"-")
        donc operation appliquée en fréquence ou en prony.

        C'est une complication inutile: il est plus facile & mnémotechnique
        de redemander 'temps' pour une autre durée.
        """
        
        self.listePcrt =  _p2.profilnormalise(self.listeP, duree)        
        
        
        if transpo == None:
            return _p.profil(self.listePcrt)
            
        else:
            valeur, operation = transpo
            p = _p.profil(self.listePcrt)
            if operation == "*":
                return p * valeur
            elif operation == "/":
                return p / valeur
            elif operation == "+":
                return p + valeur
            elif operation == "-":
                return p - valeur

    def samples(self, n, transpo=None, adapte=True):
        """
        calcule le profil-vecteur à partir du
        profil-liste de telle façon que le vecteur ait
        'n' samples.
        'transpo' : comme pour 'temps'

        if len(k) > n:
            couper == tronquer donc on obtient des profils bruiteurs;
            donc ajustement progresif.
        """        
        duree = float(n)/_sr.sr
        k = self.temps(duree, transpo)
        if not adapte:
            return k
        if len(k) > n:
            print "len(k) > n: diff", len(k) - n 
            while len(k) > n:
                diff = float(len(k) - n)/_sr.sr
                duree -= diff
                k = self.temps(duree)

        if len(k) < n:            
            d = n - len(k)
            z = NY.ones(d, NY.float64)*k[-1]
            k = NY.concatenate((k, z))
            if len(k) < n:
                print "ProfilBase.samples(n): len(resultat) < n"
        
        return k




