#-*-coding:utf-8-*-

"""evalduree.py fournit
  - les dimensions de fichiers d'un certain type dans une arborescence
  - les durées de ces fichiers selon la norme 44100 Hz/ 16 bits 

Usage : evalduree.py directory [file-type {file-type}]
Exemples :
     1. python evalduree.py -d . -t raw
     2. python evalduree.py -d informations/ -t raw -t aif
on pourrait perfectionner en permettant pls -d, wildcards
On peut créer un lien symbolique et appliquer chmod. Alors :
     evalduree -d . -t raw
"""

__author__ = "Copyright (c) René Bastian 2003-2006 <rbastian@free.fr>"
__date__ = "2003, 06.01.08"

import os
import sys
import getopt

def listefichiers(rep):
    """liste récursivement les noms des fichiers du répertoire.
    'rep' : nom de répertoire"""
    stack =[rep]
    noms =[]
    while stack:
        rep =stack.pop()
        for nom in os.listdir(rep):
            fullname =os.path.join(rep,nom)
            noms.append(fullname)
            if os.path.isdir(fullname) and not os.path.islink(fullname):
                stack.append(fullname)
    return noms

def informer(D, selection):
    """affiche les dimensions des noms X suffixes.
    'D' : liste de noms de fichiers
    'selection' : liste de suffixes (peut être vide)
    """
    sommen =0; sommed =0.0; nfichiers =0
    for nom in D:         
        if selection != []:
            for suffixe in selection:
                if nom[-len(suffixe):] in selection:
                    try:
                        n =os.stat(nom).st_size
                        duree =float(n)/(44100 * 2 * 2)
                        sommen =sommen+n
                        sommed =sommed+duree
                        print "%10.2f %14i %s" % (duree, n, nom)
                        nfichiers+=1
                    except OSError:
                        pass
        else :
           try:
                n =os.stat(nom).st_size
                duree =float(n)/(44100 * 2 * 2)
                sommen =sommen+n
                sommed =sommed+duree
                print "%10.2f %14i %s" % (duree, n, nom)
                nfichiers+=1
           except OSError:
                pass
             
    if nfichiers == 0 :
         print "Aucun fichier"
    elif nfichiers == 1 :
         print "\n%10.2f sec %10i bytes dans un fichier" % \
               (sommed, sommen)         
    else :
         heure = int(sommed) / 3600
         reste   = int(sommed) % 3600
         min = reste / 60
         sec = reste % 60
         print "\n%10.2f sec %10i bytes dans %i fichiers == %i:%i:%i" % \
          (sommed, sommen, nfichiers, heure, min, sec)
                  
def usage():
    """comment utiliser le python."""
    s ="""
    evalduree.py fournit le total des dimensions
    de fichiers d'un certain type dans une
    arborescence
    (c) René Bastian 2003    
    Usage : evalduree.py directory file-type {file-type}
    Exemples :
         evalduree.py -d . -t raw         
         evalduree.py -d informations/ -t raw -t wav
    Options :
        -d : directory
        -f : noms de fichiers
        -h : help        
        -t : type de fichier : -t raw -t py -t txt
        -a : idem que -t wav -t raw -t aif -t aiff
        -x : idem que -t raw
    """
    print s

def principal():
    """la fonction principale."""
    try:
        Options, args =getopt.getopt(sys.argv[1:],"ahd:f:t:x")
        
        
    except getopt.GetoptError:
        usage()
        sys.exit(2)
    if Options== []:
        usage()
        sys.exit(2)
    D =listefichiers(".")
    selection =[]
    for option in Options:
        if option[0]== "-x":
            selection =[".raw"]
            D =listefichiers(".")
            informer(D, selection)
            sys.exit()
        if option[0]== "-a":
            Selection =[".wav",".raw"] 
            D =listefichiers(".")
            for select in Selection:
                 
                 informer(D, [select])
            sys.exit()
        if option[0] == "-h":
            usage()
            sys.exit()
        if option[0] == "-t":
            
            selection.append(option[1])
        if option[0] == "-d":
            
            D =listefichiers(option[1])
        if option[0] == "-f":
            
            
            
            selection.append(option[1])
            
            informer(D, selection)
            print selection
            sys.exit()
            
    informer(D, selection)
        
if __name__ == "__main__":
    principal()
    


