#-*-coding:utf-8-*-

"""
fournit la classe Journal qui permet
d'afficher les noms des fichiers d'une arborescence
qui ont été modifiés ou créés depuis les n derniers jours.
"""

import os
import time

try:
    from versionneur import fairecopienumero
except ImportError:
    existe_versionneur = False
else:
    existe_versionneur = True

__author__ = "Copyright (c) René Bastian <rbastian@free.fr>"
__date__ = "2006.02.04, 08.04.20 (passage à utf-8)"

def fdate(D):
    """met en forme date et heure fournis par D."""
    return "%4i.%02i.%02i %02i:%02i:%02i" % D[:6]
    
class Journal:
    """
    classe pour collecter les noms de fichier modifiés depuis un
    certain nombre de jours.
    """
    def __init__(self, ndate, repertoire, inclus):
        """retourne une instance de la classe Journal ;
        'date' : nombre de secondes depuis 1970
        'repertoire' : racine de l'arbre de recherche
        'inclus' : suffixe des fichiers à examiner.
        """
        self.resultat = []
        self.date = ndate
        self.rep = repertoire
        self.inclus = inclus
        self.d = -len(inclus)
        
    def enregistre(self, _, rep, Liste):
        """extrait les noms de fichiers correspondant aux
        critères."""
        os.chdir(rep)
        for nom in Liste:
            if os.path.isfile(nom):
                ndate = os.stat(nom)[8]
            else:
                continue
            
            if (ndate > self.date) and (nom[self.d:] == self.inclus):
                chemin = os.path.normcase(os.path.join(rep, nom))
                self.resultat += [(ndate, chemin)]
                
    def traverse(self):
        """lance la descente dans l'arborescence."""
        os.path.walk(self.rep, self.enregistre, self.date)
        
    def __str__(self, inverser = False):
        """affichage des résultats."""
        r = "Fichiers modifiés :\n"
        self.resultat.sort()
        if inverser: 
            self.resultat.reverse()
        for (ndate, nom) in self.resultat:
            d = fdate(time.localtime(ndate))
            r += "%s %s\n" % (d, nom)
        return r

def faire(n):
    """calcule le point temporel."""
    n = long(n)
    return time.time() - float(n * 24 * 3600)

def usage():
    "une sorte d'aide"
    s = """Usage :
si le fichier n'est pas ex\'ecutable de partout :    
    python journal.py <nombre de jours> <repertoire> <suffixe>
si le fichier est ex\'ecutable de partout :    
    journal <nombre de jours> <repertoire> <suffixe>

par defaut :
    nombre de jours = 7
    repertoire = .
    suffixe = py

Exemple :
    python journal.py 15 .. html
    journal 15 ../docs tex
    journal 5 audio/raws raw
    journal"""
    print __doc__
    print s    
    
if __name__ == "__main__":
    if existe_versionneur:
        fairecopienumero(__file__)
    from optparse import OptionParser
    
    xn = 10
    un_rep = "."
    suff_inclus = ".py"
    parser = OptionParser()
    parser.add_option("-n", dest = "nombrejours",
                      help = "Combien de jours en arrière (défaut 10)")
    parser.add_option("-d", dest = "repertoire",
                      help = "Racine de l'arborescence à examiner")
    parser.add_option("-t", dest = "suffixe",
                      help = "suffixe des fichiers à sélectionner")
    parser.add_option("-r", dest = "recursif", 
                      help = "descendre dans l'arborescence (par défaut)")

    (options, args) = parser.parse_args()
    
    recursif = options.recursif
    if recursif == None:
        recursif = True
    xn = options.nombrejours
    if xn == None: 
        xn = 10    
    un_rep = options.repertoire
    if un_rep == None: 
        un_rep = "."
    if recursif: 
        un_rep = os.path.abspath(un_rep)
    suff_inclus = options.suffixe
    if suff_inclus == None: 
        suff_inclus = ".py"

    une_date = faire(xn)
    j = Journal(une_date, un_rep, suff_inclus)
    j.traverse()
    print j
    



